// MIS3502 - Web Service Template
// Created by: Jeremy Shafer
// Fall 2023

// REMINDER - Don't forget to change your database connection
// timeout from 3 seconds to 3 minutes.
// Look under Configuration / General Configuration

// declarations *****************************************
const qs = require('qs'); //for parsing URL encoded data
const axios = require('axios'); // for calling another API
const base64 = require('js-base64'); // encode / decode
const mysql = require('mysql2/promise');  //for talking to a database

const dboptions = {
  'user' : '',
  'password' : '',
  'database' : '',
  'host' : 'dataanalytics.temple.edu'
};

//global connection variable
var connection;

const features = [
	`Issue a GET against dbtest and send someid=1 as a URL encoded key/value pair in the querystring. `,
	`Issue a GET against usertest and send the key userid. 
	The JSON response will contain all user attributes that correspond to that userid.`,
	`Issue a POST against usertest and send username, password, email, usertype, firstname, lastname. 
	The JSON response will be the userid that was created by the POST.`,
	`Issue a PUT against usertest and send the userid, username, password, email, usertype, firstname, lastname.
	The corresponding user record will be updated with all the new values. 
	The JSON response will contain all attributes of the updated user record.`,
	`Issue a PATCH against changeusertype and send the userid and the usertype. The JSON response will be all attributes of the updated user record.`,
	`Issue a DELETE against usertest and provide the userid.  The corresponding user record will be deleted.
	The JSON response will be 'OK'`,	
	];


// supporting functions *********************************

let dbtest = async (res,query) => {

	// get the expected variable
	let someid = query.someid;
	
	//error trap
	if (someid == undefined || isNaN(someid)){
		return formatres(res,"The key someid is missing or incorrect.",400);
	}
	
	//work and return the result
	let [result] = await connection.execute("select now() AS the_current_time, unix_timestamp(now()) AS the_current_epoch where 1 = ?",[someid]);
	return formatres(res,result,200);
}

let getUsertest = async (res,query) => {

	let userid = query.userid;

	//error trap
	if ( userid == undefined || userid == "" || isNaN(userid) ){
		return formatres(res,"The key userid was missing or incorrect.",400);
	}

	try {
		//SQL work and return the result
		let txtSQL = 'select * from user where userid = ?'; //bug here!
		let [result] = await connection.execute(txtSQL,[userid]);
		
		//success!!
		return formatres(res,result,200);	
	} catch(e){
		return formatres(res,e,500); // good for testing
		//return formatres(res,"Unexpected Error",500); //production
	}
	
}

let postUsertest = async (res,body) => {

	let username = body.username;
	let password = body.password;
	let email = body.email;
	let usertype = body.usertype;
	let firstname = body.firstname;
	let lastname = body.lastname;

	//error trap
	if ( username == undefined || username == "" ){
		return formatres(res,"The key username was missing or incorrect.",400);
	}
	if ( password == undefined || password == ""){
		return formatres(res,"The key password was missing or incorrect.",400);
	}
	if ( email == undefined || email == "" ){
		return formatres(res,"The key email was missing or incorrect.",400);
	}
	if ( usertype == undefined || usertype == ""){
		return formatres(res,"The key usertype was missing or incorrect.",400);
	}
	if ( firstname == undefined || firstname == ""){
		return formatres(res,"The key firstname was missing or incorrect.",400);
	}
	if ( lastname == undefined || lastname == "" ){
		return formatres(res,"The key lastname was missing or incorrect.",400);
	}
	
	//SQL work and return the result
	try{
		let txtSQL = 'insert into users(username, password, email, usertype, firstname, lastname) values (?,?,?,?,?,?)';
		let [result] = await connection.execute(txtSQL,[username, password, email, usertype, firstname, lastname]);
		return formatres(res,result.insertId,200);
	} catch (e){
		//return formatres(res,"Unexpected Error",500);
		return formatres(res,e,500);
	}	
}

let putUsertest = async (res,body) => {
	
	let userid = body.userid;
	let username = body.username;
	let password = body.password;
	let email = body.email;
	let usertype = body.usertype;
	let firstname = body.firstname;
	let lastname = body.lastname;

	//error trap
	if ( userid == undefined || userid == "" || isNaN(userid) ){
		return formatres(res,"The key userid was missing or incorrect.",400);
	}
	if ( username == undefined || username == ""  ){
		return formatres(res,"The key username was missing or incorrect.",400);
	}
	if ( password == undefined || password == ""  ){
		return formatres(res,"The key password was missing or incorrect.",400);
	}
	if ( email == undefined || email == "" ){
		return formatres(res,"The key email was missing or incorrect.",400);
	}
	if ( usertype == undefined || usertype == ""  ){
		return formatres(res,"The key usertype was missing or incorrect.",400);
	}
	if ( firstname == undefined || firstname == ""  ){
		return formatres(res,"The key firstname was missing or incorrect.",400);
	}
	if ( lastname == undefined || lastname == ""  ){
		return formatres(res,"The key lastname was missing or incorrect.",400);
	}
	
	//SQL work and return the result
	try{
		let txtSQL = 'update users  set username=?, password=?, email=?, usertype=?, firstname=?, lastname=? where userid = ?';
		let [result] = await connection.execute(txtSQL,[username, password, email, usertype, firstname, lastname,userid]);
		let txtSQL2 = 'select * from users where userid = ?';
		let [result2] = await connection.execute(txtSQL2,[userid]);
		return formatres(res,result2,200);
	} catch (e){
		//return formatres(res,"Unexpected Error",500);
		return formatres(res,e,500);
	}
}

let patchUsertest = async (res,body) => {

	let userid = body.userid;
	let usertype = body.usertype;

	//error trap
	if ( userid == undefined || userid == "" || isNaN(userid) ){
		return formatres(res,"The key userid was missing or incorrect.",400);
	}
	if ( usertype == undefined || usertype == ""){
		return formatres(res,"The key usertype was missing or incorrect.",400);
	}
	
	//SQL work and return the result
	try{
		let txtSQL = 'update users set usertype=? where userid = ?';
		let [result] = await connection.execute(txtSQL,[usertype,userid]);
		let txtSQL2 = 'select * from users where userid = ?';
		let [result2] = await connection.execute(txtSQL2,[userid]);
		return formatres(res,result2,200);
	} catch (e){
		//return formatres(res,"Unexpected Error",500);
		return formatres(res,e,500);
	}
}

let delUsertest = async (res,body) => {

	let userid = body.userid;

	//error trap
	if ( userid == undefined || userid == "" || isNaN(userid) ){
		return formatres(res,"The key userid was missing or incorrect.",400);
	}
	
	
	//SQL work and return the result
	try{
		let txtSQL = 'delete users where userid = ?'; //bug here!
		let [result] = await connection.execute(txtSQL,[userid]);
		return formatres(res,"OK",200);
	} catch (e){
		//return formatres(res,"Unexpected Error",500);
		return formatres(res,e,500);
	}
}

// do not delete this handy little supporting function
let formatres = async (res, output, statusCode) => {
	
	// kill the global database connection
	if (connection != undefined &&  
		typeof(connection)=='object' &&  
		typeof(connection.end())=='object'  ){
		await connection.end();
	}

	res.statusCode = statusCode;
	res.body = JSON.stringify(output);
	return res;	
}

// do not delete this handy little supportng function
function isEmpty(obj) {
    return Object.keys(obj).length === 0;
}

// My Routing Function **********************************

let myRoutingFunction = (res,method,path,query,body) => {

	// conditional statements go here.
	// look at the path and method and return the output from the 
	// correct supporting function.

	// Simple GET request with no features specified results
	// in a list of features / instructions
	if (method == "GET" && path == ""){
		return formatres(res, features, 200);
	}

	if (method == "GET" && path == "dbtest"){
		return dbtest(res,query);
	}

	if (method == "GET" && path == "usertest"){
		return getUsertest(res,query);
	}
	
	if (method == "POST" && path == "usertest"){
		return postUsertest(res,body);
	}
	
	if (method == "PUT" && path == "usertest"){
		return putUsertest(res,body);
	}
	
	if (method == "PATCH" && path == "changeusertype"){
		return patchUsertest(res,body);
	}
	
	if (method == "DELETE" && path == "usertest"){
		return delUsertest(res,body);
	}
	
	return(res);
}


// event handler ****************************************

// Students should not have to change the code here.
// Students should be able to read and understand the code here.

exports.handler = async (request) => {

	connection = await mysql.createConnection(dboptions);	

	// identify the method (it will be a string)
	let method = request["httpMethod"];
	
	// identify the path (it will also be a string)
	let fullpath = request["path"];
	
	// we clean the full path up a little bit
	if (fullpath == undefined || fullpath == null){ fullpath = ""};
	let pathitems = fullpath.split("/");
	let path = pathitems[2];
	if (path == undefined || path == null){ path = ""};
	
	// identify the querystring ( we will convert it to 
	//   a JSON object named query)
	let query = request["queryStringParameters"];
	if (query == undefined || query == null){ query={} };
	
	// identify the body (we will convert it to 
	//   a JSON object named body)
	let body = qs.parse(request["body"]);
	if (body == undefined || body == null){ body={} };

	// Create the default response object that will include 
	// the status code, the headers needed by CORS, and
	// the string to be returned formatted as a JSON data structure.
    let res = {
        'statusCode': 400,
        'headers': {
            'Access-Control-Allow-Origin': '*',
            'Access-Control-Allow-Credentials': true  
        },
        'body': JSON.stringify("Feature not found."),
    };

	// run all the parameters through my routing function
	// and return the result
    return myRoutingFunction(res,method,path,query,body);
    
    //*** this is a good place to test one supporting function at a time
    //body={"userid":21};
    //return delUsertest(res,body);
    //query={"userid":5};
    //return getUsertest(res,query);
    
};